defmodule Nulla.Models.Follow do
  use Ecto.Schema
  import Ecto.Changeset
  alias Nulla.Repo
  alias Nulla.Snowflake
  alias Nulla.Models.Actor

  @primary_key {:id, :integer, autogenerate: false}
  schema "follows" do
    belongs_to :follower, Actor
    belongs_to :followed, Actor

    timestamps()
  end

  @doc false
  def changeset(follow, attrs) do
    follow
    |> cast(attrs, [:user_id, :target_id])
    |> validate_required([:user_id, :target_id])
    |> unique_constraint([:user_id, :target_id])
  end

  def create_follow(attrs) do
    id = Snowflake.next_id()

    %__MODULE__{}
    |> __MODULE__.changeset(attrs)
    |> Ecto.Changeset.put_change(:id, id)
    |> Repo.insert()
  end
end
